﻿#region The RIGOL Software License
/*
 * ====================================================================
 *
 * The RIGOL Software License, Version 1.0
 *
 * Copyright (c) 2014-2020 RIGOL Technoligies.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by 
 *        RIGOL Technoligies (http://www.rigol.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. Neither the name of RIGOL Technologies nor the names of its 
 *	  contributors may be used to endorse or promote products derived 
 *	  from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ====================================================================
 *
 *
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DS1000Z_Waveform
{
    class Program
    {
        /// <summary>
        /// DS1000Z_Waveform.exe visaAddr chanId length
        /// DS1000Z_Waveform.exe USB0::0x1AB1::0x04CE::DS1ZA151300002::INSTR CHAN1 12000
        /// </summary>
        /// <param name="args">args0--channel string:CHAN1/2/3/4</param>
        /// args1 -- visa addr
        /// args2 -- memory length
        static void Main(string[] args)
        {
            Int32 viDefRm, viDev, viErr;
            int i, j;
            int iRead, iEnd, length;
            CChan chan;

            // Check input
            if (args.Length < 3)
            {
                Console.WriteLine("Invalid input. eg: DS1000Z_Waveform.exe addr CHAN1 1000");
                return ;
            }

            length = Convert.ToInt32(args[2]);
            if (length < 1)
            {
                Console.WriteLine("Invalid input. eg: DS1000Z_Waveform.exe addr CHAN1 1000");
                return ;
            }

            // Open Device
            visa32.viOpenDefaultRM(out viDefRm);
            viErr = visa32.viOpen(viDefRm,args[0], 0, 20000, out viDev);
            if (viErr != visa32.VI_SUCCESS)
            {
                Console.WriteLine("Fail to open device.");
                return;
            }

            // Create chan
            chan = new CChan(args[1], viDev);

            // read and convert
            for (i = 1; i < length; i += CChan.C_PACK_LENGTH)
            {
                iEnd = i + CChan.C_PACK_LENGTH - 1;
                if (iEnd > length)
                {
                    iEnd = length;
                }
                                                    // read data
                chan.ReadData(i, iEnd, viDev);
 
                iRead = iEnd - i + 1;               // Convert to ASC
                for (j = 0; j < iRead; j++)
                {
                    Console.WriteLine(chan.CsvData(j));
                }
            }
            
            // close
            visa32.viClose( viDev );
            visa32.viClose( viDefRm );
        }
    }

    public class CChan
    {
        public const int C_PACK_LENGTH = 250000;

        int mLength;    // Data Length

        int mYRef;      // Y Ref
        int mYOr;       // Y Or
        float mYInc;    // Y Inc
        string[] mAscTable; 

        byte[] mPackBuf;    
        byte[] mTmcHeader;

        string mChanId; 

        #region property

        public string Name
        {
            get { return mChanId; }
        }

        public int Length
        {
            get { return mLength; }
        }

        public string[] AscTable
        {
            get { return mAscTable; }
        }

        #endregion

        public string CsvData(int index)
        {
            return AscTable[mPackBuf[index]];
        }

        /// <summary>
        /// Create a chan obj
        /// </summary>
        /// <param name="chanId">CHAN1,CHAN2,CHAN3,CHAN4</param>
        /// <param name="viDev">VISA device session</param>
        public CChan(string chanId, Int32 viDev)
        {
            mChanId = chanId;                   
                                                // pack buf
            mPackBuf = new Byte[C_PACK_LENGTH];
            mTmcHeader = new byte[16];          // Tmc Header

            SnapChanInfo(viDev);                // Get Chan Info
                
            BuildAscTable();                    // Create the ASC convert table
        }

        /// <summary>
        /// Get the chan Info
        /// </summary>
        /// <param name="viDev"> VISA Device Session</param>
        private void SnapChanInfo(Int32 viDev)
        {
            visa32.viPrintf(viDev, ":STOP\n");
            visa32.viPrintf(viDev, ":WAV:MODE RAW\n");
            visa32.viPrintf(viDev, ":WAV:FORM BYTE\n");
            visa32.viPrintf(viDev, ":WAV:SOUR %s\n", mChanId);

            visa32.viPrintf(viDev, ":WAV:YOR?\n");
            visa32.viScanf(viDev, "%d", out mYOr);
            visa32.viPrintf(viDev, ":WAV:YREF?\n");
            visa32.viScanf(viDev, "%d", out mYRef);

            mLength = 250000;                   // memory length

            visa32.viPrintf(viDev, ";WAV:YINC?\n");
            visa32.viScanf(viDev, "%f", out mYInc);
        }

        /// <summary>
        /// Build the Asc convert lookup table.
        /// </summary>
        private void BuildAscTable()
        {
            mAscTable = new string[256];        // 8 Bit ADC, so 0-255 total 256 items

            int i;

            for (i = 0; i < 256; i++)
            {                                   // mYRef, mYOr, mYInc depend on channel
                                                // see SnapChanInfo
                mAscTable[i] = String.Format("{0:E}", (i - mYRef - mYOr) * mYInc);
            }
        }

        /// <summary>
        /// Read the memory data from device
        /// </summary>
        /// <param name="iSta">data range from</param>
        /// <param name="iEnd">data range to</param>
        /// <param name="viDev">VISA device session</param>
        /// <returns>read count</returns>
        public int ReadData(int iSta, int iEnd, Int32 viDev)
        {
            int retCount;
            int iTmcLength;

            visa32.viPrintf(viDev, ":WAV:SOUR %s\n", mChanId);
            visa32.viPrintf(viDev, ":WAV:STOP %d\n", iEnd);
            visa32.viPrintf(viDev, ":WAV:STAR %d\n", iSta);
            visa32.viPrintf(viDev, ":WAV:DATA?\n");

            visa32.viRead(viDev, mTmcHeader, 2, out retCount);  // "#9"
            iTmcLength = mTmcHeader[1] - '0';                   // data descriptor length

                                                                // read the descriptor, eg "000001000"
            visa32.viRead(viDev, mTmcHeader, iTmcLength, out retCount);
                                                                // read the waveform bin data     
            visa32.viRead(viDev, mPackBuf, (iEnd - iSta + 1), out retCount);

            return retCount;
        }
    }
}
